@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2009 Charles Hebert <charles.hebert@logilab.fr>
 # Copyright (c) 2011-2014 Google, Inc.
@@ -17,7 +16,7 @@
 # Copyright (c) 2017 Craig Citro <craigcitro@gmail.com>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
-# Copyright (c) 2018-2020 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2018-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 # Copyright (c) 2018 Pierre Sassoulas <pierre.sassoulas@wisebim.fr>
 # Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
 # Copyright (c) 2018 Matus Valo <matusvalo@users.noreply.github.com>
@@ -29,9 +28,13 @@
 # Copyright (c) 2019 Trevor Bekolay <tbekolay@gmail.com>
 # Copyright (c) 2019 Andres Perez Hortal <andresperezcba@gmail.com>
 # Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2020 Martin Vielsmaier <martin@vielsmaier.net>
+# Copyright (c) 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2020 Damien Baty <damien.baty@polyconseil.fr>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# pylint: disable=redefined-outer-name
 
 import os
 import re
@@ -46,8 +49,7 @@ from shutil import rmtree
 
 import pytest
 
-import pylint.testutils as testutils
-from pylint import checkers, config, exceptions, interfaces, lint
+from pylint import checkers, config, exceptions, interfaces, lint, testutils
 from pylint.checkers.utils import check_messages
 from pylint.constants import (
     MSG_STATE_CONFIDENCE,
@@ -62,20 +64,14 @@ from pylint.utils import FileState, tokenize_module
 if os.name == "java":
     # pylint: disable=no-member
     # os._name is valid see https://www.programcreek.com/python/example/3842/os._name
-    if os._name == "nt":
+    if os.name == "nt":
         HOME = "USERPROFILE"
     else:
         HOME = "HOME"
+elif sys.platform == "win32":
+    HOME = "USERPROFILE"
 else:
-    if sys.platform == "win32":
-        HOME = "USERPROFILE"
-    else:
-        HOME = "HOME"
-
-try:
-    PYPY_VERSION_INFO = sys.pypy_version_info
-except AttributeError:
-    PYPY_VERSION_INFO = None
+    HOME = "HOME"
 
 
 @contextmanager
@@ -242,7 +238,7 @@ def disable():
 
 @pytest.fixture(scope="module")
 def reporter():
-    return testutils.TestReporter
+    return testutils.GenericTestReporter
 
 
 @pytest.fixture
@@ -480,7 +476,7 @@ def test_disable_alot(linter):
 
 
 def test_addmessage(linter):
-    linter.set_reporter(testutils.TestReporter())
+    linter.set_reporter(testutils.GenericTestReporter())
     linter.open()
     linter.set_current_module("0123")
     linter.add_message("C0301", line=1, args=(1, 2))
@@ -492,7 +488,7 @@ def test_addmessage(linter):
 
 
 def test_addmessage_invalid(linter):
-    linter.set_reporter(testutils.TestReporter())
+    linter.set_reporter(testutils.GenericTestReporter())
     linter.open()
     linter.set_current_module("0123")
 
@@ -534,7 +530,8 @@ def test_load_plugin_config_file():
     config_path = join(REGRTEST_DATA_DIR, "dummy_plugin.rc")
 
     run = Run(
-        ["--rcfile", config_path, join(REGRTEST_DATA_DIR, "empty.py")], exit=False,
+        ["--rcfile", config_path, join(REGRTEST_DATA_DIR, "empty.py")],
+        exit=False,
     )
     assert (
         len([ch.name for ch in run.linter.get_checkers() if ch.name == "dummy_plugin"])
@@ -569,7 +566,7 @@ def test_init_hooks_called_before_load_plugins():
 
 
 def test_analyze_explicit_script(linter):
-    linter.set_reporter(testutils.TestReporter())
+    linter.set_reporter(testutils.GenericTestReporter())
     linter.check(os.path.join(DATA_DIR, "ascript"))
     assert ["C:  2: Line too long (175/100)"] == linter.reporter.messages
 
@@ -652,11 +649,6 @@ def test_pylint_home():
         del os.environ["PYLINTHOME"]
 
 
-@pytest.mark.skipif(
-    PYPY_VERSION_INFO,
-    reason="TOX runs this test from within the repo and finds "
-    "the project's pylintrc.",
-)
 @pytest.mark.usefixtures("pop_pylintrc")
 def test_pylintrc():
     with fake_home():
@@ -767,7 +759,7 @@ def test_custom_should_analyze_file():
     wrong_file = os.path.join(package_dir, "wrong.py")
 
     for jobs in [1, 2]:
-        reporter = testutils.TestReporter()
+        reporter = testutils.GenericTestReporter()
         linter = _CustomPyLinter()
         linter.config.jobs = jobs
         linter.config.persistent = 0
@@ -785,11 +777,43 @@ def test_custom_should_analyze_file():
         assert "invalid syntax" in messages[0]
 
 
+# we do the check with jobs=1 as well, so that we are sure that the duplicates
+# are created by the multiprocessing problem.
+@pytest.mark.parametrize("jobs", [1, 2])
+def test_multiprocessing(jobs):
+    """Check that multiprocessing does not create duplicates."""
+    # For the bug (#3584) to show up we need more than one file with issues
+    # per process
+    filenames = [
+        "special_attr_scope_lookup_crash.py",
+        "syntax_error.py",
+        "unused_variable.py",
+        "wildcard.py",
+        "wrong_import_position.py",
+    ]
+
+    reporter = testutils.GenericTestReporter()
+    linter = PyLinter()
+    linter.config.jobs = jobs
+    linter.config.persistent = 0
+    linter.open()
+    linter.set_reporter(reporter)
+
+    try:
+        sys.path.append(os.path.dirname(REGRTEST_DATA_DIR))
+        linter.check([os.path.join(REGRTEST_DATA_DIR, fname) for fname in filenames])
+    finally:
+        sys.path.pop()
+
+    messages = reporter.messages
+    assert len(messages) == len(set(messages))
+
+
 def test_filename_with__init__(init_linter):
     # This tracks a regression where a file whose name ends in __init__.py,
     # such as flycheck__init__.py, would accidentally lead to linting the
     # entire containing directory.
-    reporter = testutils.TestReporter()
+    reporter = testutils.GenericTestReporter()
     linter = init_linter
     linter.open()
     linter.set_reporter(reporter)
