@@ -1,43 +1,57 @@
-# -*- coding: utf-8 -*-
 # Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Vlad Temian <vladtemian@gmail.com>
 # Copyright (c) 2014 Google, Inc.
 # Copyright (c) 2014 Arun Persaud <arun@nubati.net>
 # Copyright (c) 2015 Ionel Cristian Maries <contact@ionelmc.ro>
 # Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
-# Copyright (c) 2017 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017 Pierre Sassoulas <pierre.sassoulas@cea.fr>
+# Copyright (c) 2017, 2020 hippo91 <guillaume.peillex@gmail.com>
+# Copyright (c) 2017, 2019 Thomas Hisch <t.hisch@gmail.com>
 # Copyright (c) 2017 Daniel Miller <millerdev@gmail.com>
 # Copyright (c) 2017 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2017 Thomas Hisch <t.hisch@gmail.com>
 # Copyright (c) 2017 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2018 Sushobhit <31987769+sushobhit27@users.noreply.github.com>
 # Copyright (c) 2018 Jason Owen <jason.a.owen@gmail.com>
 # Copyright (c) 2018 Jace Browning <jacebrowning@gmail.com>
 # Copyright (c) 2018 Reverb C <reverbc@users.noreply.github.com>
+# Copyright (c) 2019-2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2019 Hugues <hugues.bruant@affirm.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 Ashley Whetter <ashley@awhetter.co.uk>
+# Copyright (c) 2020 Frank Harrison <frank@doublethefish.com>
+# Copyright (c) 2020 Matěj Grabovský <mgrabovs@redhat.com>
+# Copyright (c) 2020 Pieter Engelbrecht <pengelbrecht@rems2.com>
+# Copyright (c) 2020 Clément Pit-Claudel <cpitclaudel@users.noreply.github.com>
+# Copyright (c) 2020 Anthony Sottile <asottile@umich.edu>
 
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 # For details: https://github.com/PyCQA/pylint/blob/master/COPYING
 
+# pylint: disable=too-many-public-methods
+
 import configparser
 import contextlib
 import json
+import os
 import platform
 import re
 import subprocess
-import tempfile
+import sys
 import textwrap
+import warnings
 from io import StringIO
 from os.path import abspath, dirname, join
 from unittest import mock
 
 import pytest
 
-from pylint.constants import MAIN_CHECKER_NAME
+from pylint.constants import MAIN_CHECKER_NAME, MSG_TYPES_STATUS
 from pylint.lint import Run
 from pylint.reporters import JSONReporter
-from pylint.reporters.text import *
+from pylint.reporters.text import BaseReporter, ColorizedTextReporter, TextReporter
+from pylint.utils import utils
 
 HERE = abspath(dirname(__file__))
 CLEAN_PATH = re.escape(dirname(dirname(__file__)) + "/")
@@ -68,6 +82,10 @@ def _configure_lc_ctype(lc_ctype):
 
 class MultiReporter(BaseReporter):
     def __init__(self, reporters):
+        # pylint: disable=super-init-not-called
+        # We don't call it because there is an attribute "linter" that is set inside the base class
+        # and we have another setter here using yet undefined attribute.
+        # I don't think fixing the init order in a test class used once is worth it.
         self._reporters = reporters
         self.path_strip_prefix = os.getcwd() + os.sep
 
@@ -79,7 +97,7 @@ class MultiReporter(BaseReporter):
         for rep in self._reporters:
             rep.handle_message(msg)
 
-    def display_reports(self, layout):
+    def _display(self, layout):
         pass
 
     @property
@@ -97,7 +115,7 @@ class MultiReporter(BaseReporter):
             rep.linter = value
 
 
-class TestRunTC(object):
+class TestRunTC:
     def _runtest(self, args, reporter=None, out=None, code=None):
         if out is None:
             out = StringIO()
@@ -108,12 +126,13 @@ class TestRunTC(object):
             output = out.getvalue()
         else:
             output = None
-        msg = "expected output status %s, got %s" % (code, pylint_code)
+        msg = f"expected output status {code}, got {pylint_code}"
         if output is not None:
-            msg = "%s. Below pylint output: \n%s" % (msg, output)
+            msg = f"{msg}. Below pylint output: \n{output}"
         assert pylint_code == code, msg
 
-    def _run_pylint(self, args, out, reporter=None):
+    @staticmethod
+    def _run_pylint(args, out, reporter=None):
         args = args + ["--persistent=no"]
         with _patch_streams(out):
             with pytest.raises(SystemExit) as cm:
@@ -184,9 +203,9 @@ class TestRunTC(object):
         output = out.getvalue()
         # Get rid of the pesky messages that pylint emits if the
         # configuration file is not found.
-        pattern = r"\[{}".format(MAIN_CHECKER_NAME.upper())
+        pattern = rf"\[{MAIN_CHECKER_NAME.upper()}"
         master = re.search(pattern, output)
-        assert master is not None, "{} not found in {}".format(pattern, output)
+        assert master is not None, f"{pattern} not found in {output}"
         out = StringIO(output[master.start() :])
         parser = configparser.RawConfigParser()
         parser.read_file(out)
@@ -199,11 +218,8 @@ class TestRunTC(object):
         output = out.getvalue()
         assert "profile" not in output
 
-    def test_inexisting_rcfile(self):
-        out = StringIO()
-        with pytest.raises(IOError) as excinfo:
-            self._run_pylint(["--rcfile=/tmp/norcfile.txt"], out=out)
-        assert "The config file /tmp/norcfile.txt doesn't exist!" == str(excinfo.value)
+    def test_nonexistent_config_file(self):
+        self._runtest(["--rcfile=/tmp/this_file_does_not_exist"], code=32)
 
     def test_help_message_option(self):
         self._runtest(["--help-msg", "W0101"], code=0)
@@ -215,8 +231,7 @@ class TestRunTC(object):
         self._runtest([], code=32)
 
     def test_no_out_encoding(self):
-        """test redirection of stdout with non ascii caracters
-        """
+        """test redirection of stdout with non ascii caracters"""
         # This test reproduces bug #48066 ; it happens when stdout is redirected
         # through '>' : the sys.stdout.encoding becomes then None, and if the
         # output contains non ascii, pylint will crash
@@ -229,13 +244,19 @@ class TestRunTC(object):
         )
 
     def test_parallel_execution(self):
+        out = StringIO()
         self._runtest(
             [
                 "-j 2",
                 join(HERE, "functional", "a", "arguments.py"),
-                join(HERE, "functional", "a", "arguments.py"),
             ],
-            code=2,
+            out=out,
+            # We expect similarities to fail and an error
+            code=MSG_TYPES_STATUS["E"],
+        )
+        assert (
+            "Unexpected keyword argument 'fourth' in function call"
+            in out.getvalue().strip()
         )
 
     def test_parallel_execution_missing_arguments(self):
@@ -245,14 +266,14 @@ class TestRunTC(object):
         # Test that --py3k flag works.
         rc_code = 0
         self._runtest(
-            [join(HERE, "functional", "u", "unpacked_exceptions.py"), "--py3k"],
+            [join(HERE, "functional", "u", "unnecessary_lambda.py"), "--py3k"],
             code=rc_code,
         )
 
     def test_py3k_jobs_option(self):
         rc_code = 0
         self._runtest(
-            [join(HERE, "functional", "u", "unpacked_exceptions.py"), "--py3k", "-j 2"],
+            [join(HERE, "functional", "u", "unnecessary_lambda.py"), "--py3k", "-j 2"],
             code=rc_code,
         )
 
@@ -263,14 +284,12 @@ class TestRunTC(object):
     def test_enable_all_works(self):
         module = join(HERE, "data", "clientmodule_test.py")
         expected = textwrap.dedent(
-            """
+            f"""
         ************* Module data.clientmodule_test
-        {0}:10:8: W0612: Unused variable 'local_variable' (unused-variable)
-        {0}:18:4: C0116: Missing function or method docstring (missing-function-docstring)
-        {0}:22:0: C0115: Missing class docstring (missing-class-docstring)
-        """.format(
-                module
-            )
+        {module}:10:8: W0612: Unused variable 'local_variable' (unused-variable)
+        {module}:18:4: C0116: Missing function or method docstring (missing-function-docstring)
+        {module}:22:0: C0115: Missing class docstring (missing-class-docstring)
+        """
         )
         self._test_output(
             [module, "--disable=all", "--enable=all", "-rn"], expected_output=expected
@@ -280,12 +299,10 @@ class TestRunTC(object):
         module1 = join(HERE, "regrtest_data", "import_something.py")
         module2 = join(HERE, "regrtest_data", "wrong_import_position.py")
         expected_output = textwrap.dedent(
-            """
+            f"""
         ************* Module wrong_import_position
-        {}:11:0: C0413: Import "import os" should be placed at the top of the module (wrong-import-position)
-        """.format(
-                module2
-            )
+        {module2}:11:0: C0413: Import "import os" should be placed at the top of the module (wrong-import-position)
+        """
         )
         args = [
             module2,
@@ -330,7 +347,7 @@ class TestRunTC(object):
         assert isinstance(output[0], dict)
         # So each version wants a different column number...
         if platform.python_implementation() == "PyPy":
-            column = 8
+            column = 9
         elif sys.version_info >= (3, 8):
             column = 9
         else:
@@ -348,6 +365,7 @@ class TestRunTC(object):
             assert key in message
             assert message[key] == value
         assert "invalid syntax" in message["message"].lower()
+        assert "<unknown>" in message["message"].lower()
 
     def test_json_report_when_file_is_missing(self):
         out = StringIO()
@@ -401,12 +419,10 @@ class TestRunTC(object):
     def test_error_mode_shows_no_score(self):
         module = join(HERE, "regrtest_data", "application_crash.py")
         expected_output = textwrap.dedent(
-            """
+            f"""
         ************* Module application_crash
-        {}:1:6: E0602: Undefined variable 'something_undefined' (undefined-variable)
-        """.format(
-                module
-            )
+        {module}:1:6: E0602: Undefined variable 'something_undefined' (undefined-variable)
+        """
         )
         self._test_output([module, "-E"], expected_output=expected_output)
 
@@ -461,14 +477,12 @@ class TestRunTC(object):
         path = join(HERE, "regrtest_data", "test_pylintrc_comments.py")
         config_path = join(HERE, "regrtest_data", "comments_pylintrc")
         expected = textwrap.dedent(
-            """
+            f"""
         ************* Module test_pylintrc_comments
-        {0}:2:0: W0311: Bad indentation. Found 1 spaces, expected 4 (bad-indentation)
-        {0}:1:0: C0114: Missing module docstring (missing-module-docstring)
-        {0}:1:0: C0116: Missing function or method docstring (missing-function-docstring)
-        """.format(
-                path
-            )
+        {path}:2:0: W0311: Bad indentation. Found 1 spaces, expected 4 (bad-indentation)
+        {path}:1:0: C0114: Missing module docstring (missing-module-docstring)
+        {path}:1:0: C0116: Missing function or method docstring (missing-function-docstring)
+        """
         )
         self._test_output(
             [path, "--rcfile=%s" % config_path, "-rn"], expected_output=expected
@@ -482,11 +496,9 @@ class TestRunTC(object):
     def test_getdefaultencoding_crashes_with_lc_ctype_utf8(self):
         module = join(HERE, "regrtest_data", "application_crash.py")
         expected_output = textwrap.dedent(
-            """
-        {}:1:6: E0602: Undefined variable 'something_undefined' (undefined-variable)
-        """.format(
-                module
-            )
+            f"""
+        {module}:1:6: E0602: Undefined variable 'something_undefined' (undefined-variable)
+        """
         )
         with _configure_lc_ctype("UTF-8"):
             self._test_output([module, "-E"], expected_output=expected_output)
@@ -519,13 +531,13 @@ class TestRunTC(object):
         ],
     )
     def test_stdin(self, input_path, module, expected_path):
-        expected_output = (
-            "************* Module {module}\n"
-            "{path}:1:0: W0611: Unused import os (unused-import)\n\n"
-        ).format(path=expected_path, module=module)
+        expected_output = f"""************* Module {module}
+{expected_path}:1:0: W0611: Unused import os (unused-import)
+
+"""
 
         with mock.patch(
-            "pylint.lint._read_stdin", return_value="import os\n"
+            "pylint.lint.pylinter._read_stdin", return_value="import os\n"
         ) as mock_stdin:
             self._test_output(
                 ["--from-stdin", input_path, "--disable=all", "--enable=unused-import"],
@@ -563,10 +575,8 @@ class TestRunTC(object):
             a.join("b.py").write(b_code)
         a.join("c.py").write(c_code)
 
-        curdir = os.getcwd()
-        try:
+        with tmpdir.as_cwd():
             # why don't we start pylint in a subprocess?
-            os.chdir(str(tmpdir))
             expected = (
                 "************* Module a.b\n"
                 "a/b.py:3:0: E0401: Unable to import 'a.d' (import-error)\n\n"
@@ -581,7 +591,7 @@ class TestRunTC(object):
 
             # this code needs to work w/ and w/o a file named a/b.py on the
             # harddisk.
-            with mock.patch("pylint.lint._read_stdin", return_value=b_code):
+            with mock.patch("pylint.lint.pylinter._read_stdin", return_value=b_code):
                 self._test_output(
                     [
                         "--from-stdin",
@@ -592,8 +602,20 @@ class TestRunTC(object):
                     expected_output=expected,
                 )
 
-        finally:
-            os.chdir(curdir)
+    def test_stdin_syntaxerror(self):
+        expected_output = (
+            "************* Module a\n"
+            "a.py:1:4: E0001: invalid syntax (<unknown>, line 1) (syntax-error)"
+        )
+
+        with mock.patch(
+            "pylint.lint.pylinter._read_stdin", return_value="for\n"
+        ) as mock_stdin:
+            self._test_output(
+                ["--from-stdin", "a.py", "--disable=all", "--enable=syntax-error"],
+                expected_output=expected_output,
+            )
+            assert mock_stdin.call_count == 1
 
     def test_version(self):
         def check(lines):
@@ -608,3 +630,184 @@ class TestRunTC(object):
         result = subprocess.check_output([sys.executable, "-m", "pylint", "--version"])
         result = result.decode("utf-8")
         check(result.splitlines())
+
+    def test_fail_under(self):
+        self._runtest(
+            [
+                "--fail-under",
+                "5",
+                "--enable=all",
+                join(HERE, "regrtest_data", "fail_under_plus6.py"),
+            ],
+            code=0,
+        )
+        self._runtest(
+            [
+                "--fail-under",
+                "6",
+                "--enable=all",
+                join(HERE, "regrtest_data", "fail_under_plus6.py"),
+            ],
+            code=0,
+        )
+        self._runtest(
+            [
+                "--fail-under",
+                "5.5",
+                "--enable=all",
+                join(HERE, "regrtest_data", "fail_under_plus6.py"),
+            ],
+            code=0,
+        )
+        self._runtest(
+            [
+                "--fail-under",
+                "7",
+                "--enable=all",
+                join(HERE, "regrtest_data", "fail_under_plus6.py"),
+            ],
+            code=16,
+        )
+        self._runtest(
+            [
+                "--fail-under",
+                "6.7",
+                "--enable=all",
+                join(HERE, "regrtest_data", "fail_under_plus6.py"),
+            ],
+            code=16,
+        )
+
+        self._runtest(
+            [
+                "--fail-under",
+                "0",
+                "--enable=all",
+                join(HERE, "regrtest_data", "fail_under_minus6.py"),
+            ],
+            code=22,
+        )
+        self._runtest(
+            [
+                "--fail-under",
+                "-10",
+                "--enable=all",
+                join(HERE, "regrtest_data", "fail_under_plus6.py"),
+            ],
+            code=0,
+        )
+
+    @staticmethod
+    def test_do_not_import_files_from_local_directory(tmpdir):
+        p_astroid = tmpdir / "astroid.py"
+        p_astroid.write("'Docstring'\nimport completely_unknown\n")
+        p_hmac = tmpdir / "hmac.py"
+        p_hmac.write("'Docstring'\nimport completely_unknown\n")
+
+        with tmpdir.as_cwd():
+            subprocess.check_output(
+                [
+                    sys.executable,
+                    "-m",
+                    "pylint",
+                    "astroid.py",
+                    "--disable=import-error,unused-import",
+                ],
+                cwd=str(tmpdir),
+            )
+
+        # Linting this astroid file does not import it
+        with tmpdir.as_cwd():
+            subprocess.check_output(
+                [
+                    sys.executable,
+                    "-m",
+                    "pylint",
+                    "-j2",
+                    "astroid.py",
+                    "--disable=import-error,unused-import",
+                ],
+                cwd=str(tmpdir),
+            )
+
+        # Test with multiple jobs for hmac.py for which we have a
+        # CVE against: https://github.com/PyCQA/pylint/issues/959
+        with tmpdir.as_cwd():
+            subprocess.call(
+                [
+                    sys.executable,
+                    "-m",
+                    "pylint",
+                    "-j2",
+                    "hmac.py",
+                    "--disable=import-error,unused-import",
+                ],
+                cwd=str(tmpdir),
+            )
+
+    def test_allow_import_of_files_found_in_modules_during_parallel_check(self, tmpdir):
+        test_directory = tmpdir / "test_directory"
+        test_directory.mkdir()
+        spam_module = test_directory / "spam.py"
+        spam_module.write("'Empty'")
+
+        init_module = test_directory / "__init__.py"
+        init_module.write("'Empty'")
+
+        # For multiple jobs we could not find the `spam.py` file.
+        with tmpdir.as_cwd():
+            self._runtest(
+                [
+                    "-j2",
+                    "--disable=missing-docstring, missing-final-newline",
+                    "test_directory",
+                ],
+                code=0,
+            )
+
+        # A single job should be fine as well
+        with tmpdir.as_cwd():
+            self._runtest(
+                [
+                    "-j1",
+                    "--disable=missing-docstring, missing-final-newline",
+                    "test_directory",
+                ],
+                code=0,
+            )
+
+    def test_can_list_directories_without_dunder_init(self, tmpdir):
+        test_directory = tmpdir / "test_directory"
+        test_directory.mkdir()
+        spam_module = test_directory / "spam.py"
+        spam_module.write("'Empty'")
+
+        with tmpdir.as_cwd():
+            self._runtest(
+                [
+                    "--disable=missing-docstring, missing-final-newline",
+                    "test_directory",
+                ],
+                code=0,
+            )
+
+    def test_jobs_score(self):
+        path = join(HERE, "regrtest_data", "unused_variable.py")
+        expected = "Your code has been rated at 7.50/10"
+        self._test_output([path, "--jobs=2", "-ry"], expected_output=expected)
+
+    def test_duplicate_code_raw_strings(self):
+        path = join(HERE, "regrtest_data", "duplicate_data_raw_strings")
+        expected_output = "Similar lines in 2 files"
+        self._test_output(
+            [path, "--disable=all", "--enable=duplicate-code"],
+            expected_output=expected_output,
+        )
+
+    def test_regression_parallel_mode_without_filepath(self):
+        # Test that parallel mode properly passes filepath
+        # https://github.com/PyCQA/pylint/issues/3564
+        path = join(
+            HERE, "regrtest_data", "regression_missing_init_3564", "subdirectory/"
+        )
+        self._test_output([path, "-j2"], expected_output="No such file or directory")
